import json
import pytest
import os

from reana_jupyterlab.tests.mocks.workflows import *

@pytest.fixture
def mock_get_workflows(mocker):
    response = mocker.Mock()
    response.json.return_value = MOCK_WORKFLOWS.copy()
    response.status_code = 200

    mocker.patch('requests.get', return_value=response)

@pytest.fixture
def mock_get_workflow(mocker):
    response = mocker.Mock()

    workflow = MOCK_WORKFLOWS.copy()
    workflow['items'] = [MOCK_WORKFLOWS['items'][0]]
    workflow['total'] = 1

    response.json.return_value = workflow
    response.status_code = 200

    mocker.patch('requests.get', return_value=response)

@pytest.fixture
def mock_get_workflow_logs(mocker):
    response = mocker.Mock()
    response.json.return_value = WF1_LOGS.copy()
    response.status_code = 200

    mocker.patch('requests.get', return_value=response)

@pytest.fixture
def mock_get_workspace(mocker):
    response = mocker.Mock()
    response.json.return_value = WF1_FILES.copy()
    response.status_code = 200

    mocker.patch('requests.get', return_value=response)

@pytest.fixture
def mock_get_workspace_deleted(mocker):
    response = mocker.Mock()
    response.json.return_value = DELETED_WF_FILES.copy()
    response.status_code = 400

    mocker.patch('requests.get', return_value=response)

@pytest.fixture
def mock_get_specification(mocker):
    response = mocker.Mock()
    response.json.return_value = WF1_SPECIFICATION.copy()
    response.status_code = 200

    mocker.patch('requests.get', return_value=response)

@pytest.fixture
def mock_download_files(mocker):
    response = mocker.Mock()
    response.content = FILE_CONTENT
    response.status_code = 200

    mocker.patch('requests.get', return_value=response)
    mocker.patch('os.makedirs')
    mocker.patch('builtins.open', mocker.mock_open())

@pytest.fixture
def mock_post_validate_success(mocker):
    response = MOCK_VALIDATE_SUCCESS

    mocker.patch('subprocess.run', return_value=response)
    mocker.patch('os.path.isdir', return_value=True)

@pytest.fixture
def mock_post_validate_error(mocker):
    response = MOCK_VALIDATE_ERROR

    mocker.patch('subprocess.run', return_value=response)
    mocker.patch('os.path.isdir', return_value=True)

@pytest.fixture
def mock_post_validate_invalid_path(mocker):
    mocker.patch('os.path.isdir', return_value=False)

@pytest.fixture
def mock_post_validate_not_yaml_file(mocker):
    mocker.patch('os.path.isdir', return_value=True)

@pytest.fixture
def mock_post_create_success(mocker):
    response = MOCK_RUN_SUCCESS

    mocker.patch('subprocess.run', return_value=response)
    mocker.patch('os.path.isdir', return_value=True)

@pytest.fixture
def mock_post_create_error(mocker):
    response = MOCK_RUN_ERROR

    mocker.patch('subprocess.run', return_value=response)
    mocker.patch('os.path.isdir', return_value=True)

@pytest.fixture
def mock_post_create_invalid_path(mocker):
    mocker.patch('os.path.isdir', return_value=False)


@pytest.mark.parametrize('endpoint', ['/reana_jupyterlab/workflows'])
async def test_get_workflows(jp_fetch, endpoint, mock_get_workflows):
    params = {
        'status': 'all',
        'search': '',
        'sort': 'asc',
        'page': 1,
    }

    response = await jp_fetch(endpoint, params=params)
    data = json.loads(response.body)

    assert response.code == 200
    assert len(data['items']) == 2
    for item, mock_item in zip(data['items'], MOCK_WORKFLOWS['items']):
        assert item['id'] == mock_item['id']

@pytest.mark.parametrize('endpoint', ['/reana_jupyterlab/workflows'])
async def test_get_workflow(jp_fetch, endpoint, mock_get_workflow):
    params = {
        'workflow_id_or_name': 'mock_wf_id_1',
    }
    response = await jp_fetch(endpoint, params=params)
    data = json.loads(response.body)

    assert response.code == 200
    assert 'items' in data
    assert data.get('items')[0]['id'] == MOCK_WORKFLOWS['items'][0]['id']

@pytest.mark.parametrize('endpoint', ['/reana_jupyterlab/workflows/mock_wf_id_1/logs'])
async def test_get_workflow_logs(jp_fetch, endpoint, mock_get_workflow_logs):
    response = await jp_fetch(endpoint)
    data = json.loads(response.body)

    assert response.code == 200
    assert data == WF1_LOGS_RESPONSE

@pytest.mark.parametrize('endpoint', ['/reana_jupyterlab/workflows/mock_wf_id_1/workspace'])
async def test_get_workspace(jp_fetch, endpoint, mock_get_workspace):
    response = await jp_fetch(endpoint)
    data = json.loads(response.body)

    assert response.code == 200
    assert data == WF1_FILES_RESPONSE


@pytest.mark.parametrize('endpoint', ['/reana_jupyterlab/workflows/mock_wf_id_1/workspace'])
async def test_get_workspace_deleted(jp_fetch, endpoint, mock_get_workspace_deleted):
    response = await jp_fetch(endpoint)
    data = json.loads(response.body)

    assert response.code == 200
    assert data['message'] == DELETED_WF_FILES['message']

@pytest.mark.parametrize('endpoint', ['/reana_jupyterlab/workflows/mock_wf_id_1/specification'])
async def test_get_specification(jp_fetch, endpoint, mock_get_specification):
    response = await jp_fetch(endpoint)
    data = json.loads(response.body)

    assert response.code == 200
    assert data == WF1_SPECIFICATION

@pytest.mark.parametrize('endpoint', [f'/reana_jupyterlab/workflows/{MOCK_WF1["id"]}/workspace/{FILE_PATH}'])
async def test_download_files(jp_fetch, endpoint, mock_download_files):
    response = await jp_fetch(endpoint)
    data = json.loads(response.body)

    assert response.code == 200
    assert data['status'] == 'success'
    assert data['message'] == f'{FILE_PATH} downloaded'
    assert os.makedirs.called
    assert open.called
    assert open().write.called

@pytest.mark.parametrize('endpoint', ['/reana_jupyterlab/validate'])
async def test_post_validate_success(jp_fetch, endpoint, mock_post_validate_success):
    data = {
        'path': '/path/to/workflow.yaml'
    }

    response = await jp_fetch(endpoint, method='POST', body=json.dumps(data))
    assert response.code == 200

    data = json.loads(response.body)
    assert data.get('message', '') == 'Validation successful'
    assert data.get('status', '') == 'success'

@pytest.mark.parametrize('endpoint', ['/reana_jupyterlab/validate'])
async def test_post_validate_error(jp_fetch, endpoint, mock_post_validate_error):
    data = {
        'path': '/path/to/workflow.yaml'
    }

    response = await jp_fetch(endpoint, method='POST', body=json.dumps(data))
    assert response.code == 200

    data = json.loads(response.body)
    assert data.get('status', '') == 'error'
    assert data.get('message', '') == 'Validation failed'

@pytest.mark.parametrize('endpoint', ['/reana_jupyterlab/validate'])
async def test_post_validate_invalid_path(jp_fetch, endpoint, mock_post_validate_invalid_path):
    data = {
        'path': '/path/to/workflow.yaml'
    }

    response = await jp_fetch(endpoint, method='POST', body=json.dumps(data))
    assert response.code == 200

    data = json.loads(response.body)
    assert data.get('status', '') == 'error'
    assert data.get('message', '') == 'Invalid path'

@pytest.mark.parametrize('endpoint', ['/reana_jupyterlab/validate'])
async def test_post_validate_not_yaml_file(jp_fetch, endpoint, mock_post_validate_not_yaml_file):
    data = {
        'path': '/path/to/workflow.txt'
    }

    response = await jp_fetch(endpoint, method='POST', body=json.dumps(data))
    assert response.code == 200

    data = json.loads(response.body)
    assert data.get('status', '') == 'error'
    assert data.get('message', '') == 'Invalid path'

@pytest.mark.parametrize('endpoint', ['/reana_jupyterlab/run'])
async def test_post_create_success(jp_fetch, endpoint, mock_post_create_success):
    data = {
        'name': 'workflow',
        'path': '/path/to/workflow.yaml'
    }

    response = await jp_fetch(endpoint, method='POST', body=json.dumps(data))
    assert response.code == 200

    data = json.loads(response.body)
    assert data.get('message', '') == 'Workflow created'
    assert data.get('status', '') == 'success'

@pytest.mark.parametrize('endpoint', ['/reana_jupyterlab/run'])
async def test_post_create_error(jp_fetch, endpoint, mock_post_create_error):
    data = {
        'name': 'workflow',
        'path': '/path/to/workflow.yaml'
    }

    response = await jp_fetch(endpoint, method='POST', body=json.dumps(data))
    assert response.code == 200

    data = json.loads(response.body)
    assert data.get('status', '') == 'error'
    assert data.get('message', '') == 'Workflow not created'

@pytest.mark.parametrize('endpoint', ['/reana_jupyterlab/run'])
async def test_post_create_invalid_path(jp_fetch, endpoint, mock_post_create_invalid_path):
    data = {
        'name': 'workflow',
        'path': '/path/to/workflow.yaml'
    }

    response = await jp_fetch(endpoint, method='POST', body=json.dumps(data))
    assert response.code == 200

    data = json.loads(response.body)
    assert data.get('status', '') == 'error'
    assert data.get('message', '') == 'Invalid path'

@pytest.mark.parametrize('endpoint', ['/reana_jupyterlab/run'])
async def test_post_create_invalid_name(jp_fetch, endpoint, mock_post_create_success):
    data = {
        'name': 'workflow!?@',
        'path': '/path/to/workflow.yaml'
    }

    response = await jp_fetch(endpoint, method='POST', body=json.dumps(data))
    assert response.code == 200

    data = json.loads(response.body)
    assert data.get('status', '') == 'error'
    assert data.get('message', '') == 'Invalid workflow name'


from jupyter_server.base.handlers import APIHandler
import os
import json
import re
import requests
import subprocess
from urllib.parse import quote_plus, urlencode

# import ../const.py file
from ..const import (
    WORKFLOWS_PAGE_SIZE,
    WORKFLOWS_TYPE,
    WORKSPACE_PAGE_SIZE
)

endpoint = 'workflows'

class WorkflowsHandler(APIHandler):
    def _parse_workflow(self, workflow):
        parsed_workflow = {}

        parsed_workflow['id'] = workflow.get('id', '')
        info = workflow.get('name', '').rsplit('.', 1)
        parsed_workflow['name'], parsed_workflow['run'] = info
        parsed_workflow['status'] = workflow.get('status')
        parsed_workflow['createdAt'] = workflow.get('created')
        parsed_workflow['startedAt'] = workflow.get('progress').get('run_started_at', '')
        parsed_workflow['finishedAt'] = workflow.get('progress').get('run_finished_at', '')
        parsed_workflow['stoppedAt'] = workflow.get('progress').get('run_stopped_at', '')
        parsed_workflow['finishedJobs'] = workflow.get('progress').get('finished', {}).get('total', 0)
        parsed_workflow['totalJobs'] = workflow.get('progress').get('total', {}).get('total', 0)

        return parsed_workflow
    
    def _parse_workflows(self, workflows):
        parsed_workflows = []

        data = workflows.json()

        for workflow in data['items']:
            parsed_workflows.append(self._parse_workflow(workflow))

        data['items'] = parsed_workflows

        return data

    def _parse_params(self, params):
        params = {key: params[key][0].decode('utf-8') for key in params if not key.isdigit()}

        if 'status' in params and params['status'] == 'all':
            del params['status']

        if 'search' in params:
            params['search'] = json.dumps({'name': [params['search']]})

        params['access_token'] = os.getenv('REANA_ACCESS_TOKEN', '')
        params['type'] = WORKFLOWS_TYPE
        params['size'] = WORKFLOWS_PAGE_SIZE
        params['include_progress'] = True

        string_params = urlencode(params, quote_via=quote_plus)

        return string_params

    def get(self):
        params = self.request.query_arguments
        string_params = self._parse_params(params)
        server_url = os.getenv('REANA_SERVER_URL', '')

        try:
            response = requests.get(f"{server_url}/api/{endpoint}?{string_params}")
            workflows = self._parse_workflows(response)
            self.finish(json.dumps(workflows))
        except Exception as e:
            self.finish(json.dumps({
                'status': 'error',
                'message': str(e)
            }))

class WorkflowLogsHandler(APIHandler):
    def _parse_logs(self, workflow):
        wf = workflow.json()
        logs = json.loads(wf.get('logs', ''))
        return json.dumps(
            {
                'engineLogs': logs['workflow_logs'],
                'jobLogs': logs['job_logs']
            }
        )
    
    def get(self, workflow_id):
        server_url = os.getenv('REANA_SERVER_URL', '')
        access_token = os.getenv('REANA_ACCESS_TOKEN', '')

        try:
            response = requests.get(f"{server_url}/api/{endpoint}/{workflow_id}/logs?access_token={access_token}")
            logs = self._parse_logs(response)
            self.finish(logs)

        except Exception as e:
            self.finish(json.dumps({
                'status': 'error',
                'message': str(e)
            }))

class WorkflowWorkspaceHandler(APIHandler):
    def _parse_files(self, files):
        parsed_files = []

        for file in files:
            parsed_files.append(
                {
                    'name': file.get('name'),
                    'lastModified': file.get('last-modified'),
                    'size': file.get('size').get('human_readable'),
                }
            )

        return parsed_files
    
    def _parse_params(self, params):
        params = {key: params[key][0].decode('utf-8') for key in params if not key.isdigit()}

        if 'search' in params:
            params['search'] = json.dumps({'name': [params['search']]})

        params['access_token'] = os.getenv('REANA_ACCESS_TOKEN', '')
        params['size'] = WORKSPACE_PAGE_SIZE

        string_params = urlencode(params, quote_via=quote_plus)

        return string_params
    
    def get(self, workflow_id):
        params = self.request.query_arguments
        string_params = self._parse_params(params)
        server_url = os.getenv('REANA_SERVER_URL', '')

        try:
            response = requests.get(f"{server_url}/api/{endpoint}/{workflow_id}/workspace?{string_params}")
            data = response.json()

            if response.status_code != 200:
                raise Exception(data.get('message', 'Error getting workspace files'))
            
            data['files'] = self._parse_files(data.pop('items'))
            self.finish(data)
        except Exception as e:
            self.finish(json.dumps({
                'status': 'error',
                'message': str(e),
            }))

class WorkflowSpecificationHandler(APIHandler):
    def get(self, workflow_id):
        server_url = os.getenv('REANA_SERVER_URL', '')
        access_token = os.getenv('REANA_ACCESS_TOKEN', '')

        try:
            response = requests.get(f"{server_url}/api/{endpoint}/{workflow_id}/specification?access_token={access_token}")
            self.finish(response.json())
        except Exception as e:
            self.finish(json.dumps({
                'status': 'error',
                'message': str(e)
            }))

class WorkspaceFilesHandler(APIHandler):
    def get(self, workflow_name, file_name):
        server_url = os.getenv('REANA_SERVER_URL', '')
        access_token = os.getenv('REANA_ACCESS_TOKEN', '')

        try:
            file = quote_plus(file_name)
            response = requests.get(f"{server_url}/api/{endpoint}/{workflow_name}/workspace/{file}?access_token={access_token}")

            path = file_name.rsplit('/', 1)
            path = path[0] if len(path) > 1 else ''
            
            os.makedirs(workflow_name + '/' + path, exist_ok=True)

            with open(workflow_name + '/' + file_name, 'wb') as f:
                f.write(response.content)
            self.finish(json.dumps({
                'status': 'success',
                'message': f'{file_name} downloaded'
            }))
        except Exception as e:
            self.finish(json.dumps({
                'status': 'error',
                'message': str(e)
            }))


class WorkflowCreateHandler(APIHandler):
    def post(self):
        try:
            body = json.loads(self.request.body)

            wf_name = body.get('name')

            path = os.path.join(os.getcwd(), body.get('path'))
            path_split = path.rsplit('/', 1)
            workspace, yaml_file = path_split

            if '..' in path or not os.path.isdir(workspace) or not yaml_file.endswith('.yaml'):
                raise Exception('Invalid path')
            
            # Check that the workflow name does not have characters that may cause issues
            if re.fullmatch(r'\w+', wf_name) is None:
                raise Exception('Invalid workflow name')
            
            result = subprocess.run(['reana-client', 'run', '-w', wf_name, '-f', yaml_file], cwd=workspace, capture_output=True)

            if result.returncode != 0:
                raise Exception(result.stderr.decode('utf-8'))
            
            self.finish(json.dumps({
                'status': 'success',
                'message': 'Workflow created'
            }))

        except Exception as e:
            self.finish(json.dumps({
                'status': 'error',
                'message': str(e)
            }))

class WorkflowValidateHandler(APIHandler):
    def post(self):
        try:
            body = json.loads(self.request.body)
            path = os.path.join(os.getcwd(), body.get('path'))

            path_split = path.rsplit('/', 1)
            workspace, yaml_file = path_split

            if '..' in path or not os.path.isdir(workspace) or not yaml_file.endswith('.yaml'):
                raise Exception('Invalid path')
            
            result = subprocess.run(['reana-client', 'validate', '-f', yaml_file], cwd=workspace, capture_output=True)

            if result.returncode != 0:
                raise Exception(result.stderr.decode('utf-8'))
            
            self.finish(json.dumps({
                'status': 'success',
                'message': result.stdout.decode('utf-8')
            }))

        except Exception as e:
            self.finish(json.dumps({
                'status': 'error',
                'message': str(e)
            }))


MOCK_WF1 = {
    "id": "mock_wf_id_1",
    "name": "mock_wf_name.1",
    "status": "running",
    "created": "2020-10-10T10:10:10",
    "progress": {},
}

MOCK_WF2 = {
    "id": "mock_wf_id_2",
    "name": "mock_wf_name.2",
    "status": "finished",
    "created": "2020-10-10T10:10:10",
    "progress": {},
}

MOCK_WORKFLOWS = {
    "items": [MOCK_WF1, MOCK_WF2],
}

WF1_LOGS = {
    "logs": '{"workflow_logs":"engine_logs","job_logs":[{"step_1_id":{},"step_2_id":{}}]}'    
}

WF1_LOGS_RESPONSE = {
    "engineLogs": "engine_logs",
    "jobLogs": [{"step_1_id": {}, "step_2_id": {}}]
}

WF1_FILES = {
    "items": [
        {
            'name': 'file1',
            'last-modified': '2020-10-10T10:10:10',
            'size': {'human_readable': '10MB'},
        },
        {
            'name': 'file2',
            'last-modified': '2020-10-10T10:10:10',
            'size': {'human_readable': '20MB'},
        },
    ]
}

DELETED_WF_FILES = {
    "message": "Workflow workspace was deleted"
}

WF1_FILES_RESPONSE = {
    "files": [
        {
            'name': 'file1',
            'lastModified': '2020-10-10T10:10:10',
            'size': '10MB',
        },
        {
            'name': 'file2',
            'lastModified': '2020-10-10T10:10:10',
            'size': '20MB',
        },
    ]
}

WF1_SPECIFICATION = {
    "parameters": {"param1": "value1"},
    "specification": {"jobs": "job1"},
}

FILE_PATH = 'file1'
FILE_CONTENT = b'file1 content'

from subprocess import CompletedProcess

MOCK_VALIDATE_SUCCESS = CompletedProcess(
    args=['reana-client', 'validate', '-f', 'workflow.yaml'],
    returncode=0,
    stdout=b'Validation successful',
    stderr=b'',
)

MOCK_VALIDATE_ERROR = CompletedProcess(
    args=['reana-client', 'validate', '-f', 'workflow.yaml'],
    returncode=1,
    stdout=b'',
    stderr=b'Validation failed',
)

MOCK_RUN_SUCCESS = CompletedProcess(
    args=['reana-client', 'run', '-w', 'workflow', '-f', 'workflow.yaml'],
    returncode=0,
    stdout=b'Workflow created',
    stderr=b'',
)

MOCK_RUN_ERROR = CompletedProcess(
    args=['reana-client', 'run', '-w', 'workflow', '-f', 'workflow.yaml'],
    returncode=1,
    stdout=b'',
    stderr=b'Workflow not created',
)



import json
import pytest
from pathlib import Path
import shutil

ENDPOINT = '/reana_jupyterlab/files'

@pytest.fixture
def filespace():
    """
    test_workspace/
        folder1/
            file1.txt
            folder2/
                file2.csv
                file3.yaml
                folder3/
        file4.java
        file5.py
        file6.yaml
    """

    base_path = Path("./pytest_workspace").resolve()

    if base_path.exists():
        shutil.rmtree(base_path)
     
    folder3 = base_path / "folder1" / "folder2" / "folder3"
    folder3.mkdir(parents=True)

    # files
    (base_path / "file4.java").touch()
    (base_path / "file5.py").touch()
    (base_path / "file6.yaml").touch()
    (base_path / "folder1" / "file1.txt").touch()
    (base_path / "folder1" / "folder2" / "file2.csv").touch()
    (base_path / "folder1" / "folder2" / "file3.yaml").touch()

    yield base_path

    shutil.rmtree(base_path)

@pytest.mark.parametrize('path, expected', [
    ('pytest_workspace', set([('folder1', 'directory', 'pytest_workspace/folder1'), ('file6.yaml', 'file', 'pytest_workspace/file6.yaml')])),
    ('pytest_workspace/folder1', set([('folder2', 'directory', 'pytest_workspace/folder1/folder2')])),
    ('pytest_workspace/folder1/folder2', set([('file3.yaml', 'file', 'pytest_workspace/folder1/folder2/file3.yaml'), ('folder3', 'directory', 'pytest_workspace/folder1/folder2/folder3')])),
    ('pytest_workspace/folder1/folder2/folder3', set()),
    ('pytest_workspace/folder4', set()),
    ('pytest_workspace/file4.java', set()),
    ('pytest_workspace/folder1/../file6.yaml', set())
])
async def test_get_files(jp_fetch, path, expected, filespace):
    try:
        response = await jp_fetch(ENDPOINT, params={'path': path})
        assert response.code == 200

        data = json.loads(response.body)

        assert 'entries' in data
        entries = data['entries']

        assert isinstance(entries, list)

        received = set((entry['name'], entry['type'], entry['path']) for entry in entries)
        assert received == expected

        
    except Exception as e:
        print(e)
        assert e.code == 404


from jupyter_server.base.handlers import APIHandler
import os
import json
import requests

endpoint = 'you'
class EnvVariablesHandler(APIHandler):
    def _update_env(self, access_token='', server=''):
        os.environ['REANA_SERVER_URL'] = server
        os.environ['REANA_ACCESS_TOKEN'] = access_token  
    
    def get(self):
        server = os.getenv('REANA_SERVER_URL', '')
        access_token = os.getenv('REANA_ACCESS_TOKEN', '')

        self.finish(json.dumps({
            'server': server,
            'accessToken': access_token
        }))
        
    def post(self):
        data = self.get_json_body()

        try:
            server = data.get('server', '')
            access_token = data.get('accessToken', '')

            self._update_env(access_token, server)
            response = requests.get(f"{server}/api/{endpoint}?access_token={access_token}")

            data = response.json()
            if response.status_code != 200 or 'reana_server_version' not in data:
                self._update_env()
                self.finish(json.dumps({
                    'status': 'error',
                    'message': f'Could not connect to the REANA server. {data.get("message", "Server not valid.").capitalize()}'
                }))
            else:
                self.finish(json.dumps({
                    'status': 'success',
                    'message': 'Credentials saved successfully. Please close any running terminals to apply the changes.'
                }))
                

        except Exception as e:
            print(e)
            self.finish(json.dumps({
                'status': 'error',
                'message': 'Could not connect to the REANA server. Please check the server URL and access token.'
            }))


import os
from jupyter_server.base.handlers import APIHandler

class FileBrowserHandler(APIHandler):
    async def get(self):
        relative_path = self.get_query_argument('path', '')
        path = os.path.join(os.getcwd(), relative_path)

        if '..' in path or not os.path.isdir(path):
            self.set_status(404)
            self.finish({"error": "Directory not found"})
            return

        entries = []
        for entry in os.listdir(path):
            entry_path = os.path.join(path, entry)
            if not entry.startswith('.') and (os.path.isdir(entry_path) or entry.endswith('.yaml')):
                rel_entry_path = os.path.join(relative_path, entry)
                entries.append({
                    "name": entry,
                    "type": "directory" if os.path.isdir(entry_path) else "file",
                    "path": rel_entry_path,
                })

        self.finish({"entries": entries})


from jupyter_server.utils import url_path_join
from jupyter_server.serverapp import ServerApp
from .connection import EnvVariablesHandler
from .files import FileBrowserHandler
from .workflows import (
    WorkflowsHandler,
    WorkflowLogsHandler, 
    WorkflowWorkspaceHandler, 
    WorkflowSpecificationHandler,
    WorkspaceFilesHandler,
    WorkflowCreateHandler,
    WorkflowValidateHandler,
)

def setup_handlers(server_app: ServerApp) -> None:
    web_app = server_app.web_app
    host_pattern = ".*$"
    base_url = url_path_join(web_app.settings["base_url"], "reana_jupyterlab")
    handlers = [
        (url_path_join(base_url, "env"), EnvVariablesHandler),
        (url_path_join(base_url, "workflows", "([^/]+)", "workspace", "([^/]+)"), WorkspaceFilesHandler),
        (url_path_join(base_url, "workflows", "([^/]+)", "workspace"), WorkflowWorkspaceHandler),
        (url_path_join(base_url, "workflows", "([^/]+)", "logs"), WorkflowLogsHandler),
        (url_path_join(base_url, "workflows", "([^/]+)", "specification"), WorkflowSpecificationHandler),
        (url_path_join(base_url, "workflows"), WorkflowsHandler),
        (url_path_join(base_url, "run"), WorkflowCreateHandler),
        (url_path_join(base_url, "validate"), WorkflowValidateHandler),
        (url_path_join(base_url, "files"), FileBrowserHandler),
    ]
    web_app.add_handlers(host_pattern, handlers)


import pytest
from reana_jupyterlab.handlers.connection import EnvVariablesHandler

pytest_plugins = ["pytest_jupyter.jupyter_server"]

@pytest.fixture
def jp_server_config():
    return {
        "ServerApp": {
            "jpserver_extensions": {"reana_jupyterlab": True},
        }
    }



import json
import pytest

from reana_jupyterlab.tests.mocks.connection import *


@pytest.fixture
def mock_get_env_vars(mocker):
    mocker.patch('os.environ.get', return_value='mock_value')

@pytest.fixture
def mock_post_env_vars_success(mocker):
    response = mocker.Mock()
    response.json.return_value = {
        "reana_server_version": "0.7.0"
    }

    response.status_code = 200

    mocker.patch('requests.get', return_value=response)

@pytest.fixture
def mock_post_env_vars_failure(mocker):
    response = mocker.Mock()
    response.status_code = 400

    mocker.patch('requests.get', return_value=response)

@pytest.mark.parametrize('endpoint', ['/reana_jupyterlab/env'])
async def test_get_env_vars(jp_fetch, endpoint, mock_get_env_vars):
    response = await jp_fetch(endpoint)
    assert response.code == 200
    assert response.headers['Content-Type'] == 'application/json'

    data = json.loads(response.body)

    assert data['server'] == 'mock_value'
    assert data['accessToken'] == 'mock_value'

@pytest.mark.parametrize('endpoint', ['/reana_jupyterlab/env'])
async def test_post_env_vars_success(jp_fetch, endpoint, mock_post_env_vars_success):
    data = {
        'server': MOCK_SERVER,
        'accessToken': MOCK_TOKEN
    }

    response = await jp_fetch(endpoint, method='POST', body=json.dumps(data))
    assert response.code == 200

    data = json.loads(response.body)

    assert data.get('status', '') == 'success'
    assert data.get('message', '').startswith('Credentials saved successfully')


@pytest.mark.parametrize('endpoint', ['/reana_jupyterlab/env'])
async def test_post_env_vars_failure(jp_fetch, endpoint, mock_post_env_vars_failure):
    data = {
        'server': MOCK_SERVER,
        'accessToken': 'not_a_valid_token'
    }

    response = await jp_fetch(endpoint, method='POST', body=json.dumps(data))
    assert response.code == 200

    data = json.loads(response.body)
    assert data.get('status', '') == 'error'
    assert data.get('message', '').startswith('Could not connect to the REANA server')


# Copyright European Organization for Nuclear Research (CERN)
#
# Licensed under the Apache License, Version 2.0 (the "License");
# You may not use this file except in compliance with the License.
# You may obtain a copy of the License at http://www.apache.org/licenses/LICENSE-2.0
#
# Authors:
# - Muhammad Aditya Hilmy, <mhilmy@hey.com>, 2020

from .handlers import setup_handlers


def _load_jupyter_server_extension(lab_app):  # pragma: no cover
    """Registers the API handler to receive HTTP requests from the frontend extension.

    Parameters
    ----------
    lab_app: jupyterlab.labapp.LabApp
        JupyterLab application instance
    """
    setup_handlers(lab_app)
    lab_app.log.info("Registered Reana JupyterLab extension at URL path /reana_jupyterlab")